/*
 * (c) 2011 Mathews Lab, University of Rochester Medical Center.
 *
 * This software is part of a group specifically designed for the RNAstructure
 * secondary structure prediction program and its related applications.
 */

package ur_rna.RNAstructureUI.utilities;

import javafx.scene.input.KeyCode;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.tools.Tool;

import static ur_rna.Utilities.ObjTools.isEmpty;

/**
 * A class that creates a menu.
 *
 * @author Jessica S. Reuter
 */
public abstract class RNAstructureMenu
	extends JMenu implements ActionListener {
	private static final long serialVersionUID = 20120802;
	private int defaultKeyStrokeModifier = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(); //on Windows and Linux, this is Event.CTRL_MASK. On Mac, this is Event.META_MASK
	/**
	 * Constructor.
	 *
	 * @param title   The title of the menu.
	 */
	protected RNAstructureMenu( String title ) {
		setText( title );
	}

	@Override
	public void actionPerformed( ActionEvent e ) {
		doMenuActions( e.getActionCommand() );
	}

	/**
	 * Add a check box menu item with rollover text.
	 *
	 * @param text       The menu item text.
	 * @param rollover   The rollover text.
	 */
	protected JCheckBoxMenuItem addCheckBoxMenuItem(
			String text, String rollover ) {
		return addCheckBoxMenuItem(text, rollover, false);
	}
	protected JCheckBoxMenuItem addCheckBoxMenuItem(
		String text, String rollover, boolean checked ) {
		JCheckBoxMenuItem item = new JCheckBoxMenuItem( text );
		item.setName( text );
		item.setSelected( checked );
		item.setActionCommand( text );
		item.addActionListener( this );
		item.setToolTipText( rollover );
		add( item );
		return item;
	}

	/**
	 * Add a plain text menu item with rollover text.
	 *
	 * @param text       The menu item text.
	 * @param rollover   The rollover text.
	 */
	protected JMenuItem addMenuItem( String text, String rollover ) {
		JMenuItem item = new JMenuItem( text );
		item.setName( text );
		item.setActionCommand( text );
		item.addActionListener( this );
		item.setToolTipText( rollover );
		add( item );
		return item;
	}

	/**
	 * Add a plain text menu item with rollover text and a key stroke which
	 * can activate it.
	 *
	 * @param text       The menu item text.
	 * @param rollover   The rollover text.
	 * @param key        The key stroke.
	 */
	protected JMenuItem addMenuItem(
		String text, String rollover, char key ) {
		JMenuItem item = addMenuItem( text, rollover );
		KeyStroke stroke = KeyStroke.getKeyStroke( key, defaultKeyStrokeModifier );
		item.setAccelerator( stroke );
		return item;
	}

	/**
	 * Add a plain text menu item with rollover text and a key stroke which
	 * can activate it.
	 *
	 * @param text       The menu item text.
	 * @param rollover   The rollover text.
	 * @param key        The key stroke.
	 */
	protected JMenuItem addMenuItem(String text, String rollover, String key ) {
		JMenuItem item = addMenuItem( text, rollover );
		item.setAccelerator( getKey( key ) );
		return item;
	}

	protected KeyStroke getKey(String key) {
		if (isEmpty(key))
			return null;
		int modifier = 0;
		for (int i = 0; i < key.length(); i++) {
			char c = key.charAt(i);
			switch (c) {
				case '*':
					modifier |= defaultKeyStrokeModifier;
					break;
				case '^':
					modifier |= Event.CTRL_MASK;
					break;
				case '#':
					modifier |= Event.META_MASK;
					break;
				case '+':
					modifier |= Event.SHIFT_MASK;
					break;
				case '%':
					modifier |= Event.ALT_MASK;
					break;
				default:
					key = key.substring(i); // removes previous modifiers
					int code = KeyStroke.getKeyStroke(key).getKeyCode();
					//noinspection MagicConstant
					return KeyStroke.getKeyStroke(code, modifier);
			}
		}
		// We shouldn't get here unless the only characters in the string were modifier characters.
		return KeyStroke.getKeyStroke(modifier, 0);
	}

	/**
	 * Do any actions specific to this menu.
	 *
	 * @param command   The command that signifies a particular action.
	 */
	protected abstract void doMenuActions( String command );

	/**
	 * Insert a plain text menu item with rollover text.
	 * <br><br>
	 * This method is used to add window-specific menu items and actions to an
	 * already existing menu. Note that any menu item added this way must have
	 * its actions defined in the window it acts upon.
	 *
	 * @param text       The menu item text.
	 * @param rollover   The rollover text.
	 * @param index      The index at which to insert the item, one-indexed.
	 * @param listener   The action listener attached to this item.
	 */
	public JMenuItem insertMenuItem(
		String text, String rollover, ActionListener listener, int index ) {
		JMenuItem item = new JMenuItem( text );
		item.setName( text );
		item.setActionCommand( text );
		item.addActionListener( listener );
		item.setToolTipText( rollover );
		insert( item, index - 1 );
		return item;
	}
}
